/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// GrabbedImageDialog.cpp : implementation file
//

#include "stdafx.h"
#include "GrabbedImageDialog.h"
#include "Display.h"
#include "Lafter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGrabbedImageDialog

CGrabbedImageDialog::CGrabbedImageDialog(CWnd *pParent)
: CStaticFrameBBDialog( pParent )
{
	m_bCaptured = FALSE;
	m_bFirstBox = TRUE;
	m_rectBox.SetRect(0,0,0,0);
	m_picLiveImg.UnsetDrawBB();
}


CGrabbedImageDialog::CGrabbedImageDialog(CDialog *pDlg)
: CStaticFrameBBDialog( pDlg )
{
	m_bCaptured = FALSE;
	m_bFirstBox = TRUE;
	m_rectBox.SetRect(0,0,0,0);
	m_picLiveImg.UnsetDrawBB();
}

CGrabbedImageDialog::~CGrabbedImageDialog()
{
}

BEGIN_MESSAGE_MAP(CGrabbedImageDialog, CDialog)
//{{AFX_MSG_MAP(CGrabbedImageDialog)
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_PAINT()
ON_WM_CLOSE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGrabbedImageDialog message handlers

void CGrabbedImageDialog::DrawNewRect(CRect* rectBox, CPoint pointMouse, BOOL MoveBox)
{
	WINDOWPLACEMENT  wPlacement;
	
	CDC* pControlDC = m_picLiveImg.GetDC();
	// Get the position of the pic relative to the Grabbed Image Dialog
	m_picLiveImg.GetWindowPlacement( &wPlacement );
	
	CRect ClientRect;
	CRgn ClientRgn;
	// Retrieve the size of the pic 
	m_picLiveImg.GetClientRect(ClientRect);
	
	CRect ImgRect;
	CRect ParentRect;
	CPoint Offset = CPoint( wPlacement.rcNormalPosition.top, wPlacement.rcNormalPosition.left );
	// Retrieve the position of the windows
	m_picLiveImg.GetWindowRect(ImgRect);
	GetWindowRect(ParentRect);
	
	// Add the Offset to the pointMouse
	pointMouse -= Offset;
	
	pControlDC->LPtoDP(ClientRect);
	ClientRgn.CreateRectRgnIndirect(ClientRect);
	
	if (ClientRgn.PtInRegion(pointMouse)) {
		CRect tlClientRect, trClientRect, blClientRect, brClientRect;
		CRgn tlClientRgn, trClientRgn, blClientRgn, brClientRgn;
		
		int nXMid, nYMid, nXFull, nYFull;
		
		nXFull = ClientRect.Width();
		nXMid = rectBox->CenterPoint().x;
		nYFull = ClientRect.Height();
		nYMid = rectBox->CenterPoint().y;
		
		tlClientRect.SetRect(0,0,nXMid, nYMid);
		trClientRect.SetRect(nXMid, 0, nXFull, nYMid);
		blClientRect.SetRect(0,nYMid, nXMid, nYFull);
		brClientRect.SetRect(nXMid, nYMid, nXFull, nYFull);
		
		pControlDC->LPtoDP(tlClientRect);
		pControlDC->LPtoDP(trClientRect);
		pControlDC->LPtoDP(blClientRect);
		pControlDC->LPtoDP(brClientRect);
		
		tlClientRgn.CreateRectRgnIndirect(tlClientRect);
		trClientRgn.CreateRectRgnIndirect(trClientRect);
		blClientRgn.CreateRectRgnIndirect(blClientRect);
		brClientRgn.CreateRectRgnIndirect(brClientRect);
		
		CPoint pointTopLeft = rectBox->TopLeft();
		CPoint pointBottomRight = rectBox->BottomRight();
		
		if (tlClientRgn.PtInRegion(pointMouse)) {
			pointTopLeft = pointMouse;
		} else if (trClientRgn.PtInRegion(pointMouse)) {
			pointBottomRight.x = pointMouse.x;
			pointTopLeft.y = pointMouse.y;
		} else if (blClientRgn.PtInRegion(pointMouse)) {
			pointTopLeft.x = pointMouse.x;
			pointBottomRight.y = pointMouse.y;
		} else if (brClientRgn.PtInRegion(pointMouse)) {
			pointBottomRight = pointMouse;
		}
		
		pControlDC->DPtoLP(&pointTopLeft);
		pControlDC->DPtoLP(&pointBottomRight);
		rectBox->SetRect(pointTopLeft.x, pointTopLeft.y, pointBottomRight.x, pointBottomRight.y);
	}
	
	pControlDC->DPtoLP(&pointMouse);
	m_pointMouse = pointMouse;
	pControlDC->LPtoDP(rectBox);
	m_picLiveImg.ReleaseDC(pControlDC);
	
	m_picLiveImg.SetRect(*rectBox);
	m_picLiveImg.SetDrawBB();
	m_picLiveImg.m_bValidDib=CDisplay::ONCE;
}

void CGrabbedImageDialog::OnLButtonDown(UINT nFlags, CPoint point) 
{
	
	// TODO: Add your message handler code here and/or call default
	m_bMoveBox = FALSE;
	
	if (m_bFirstBox)
	{
		m_rectBox.SetRect(point.x,point.y,point.x,point.y);
		m_bFirstBox = FALSE;
	}
	else
	{
		DrawNewRect(&m_rectBox, point);	
	}
	SetCapture();
	m_bCaptured = TRUE;
	
	::SetCursor(::LoadCursor(NULL, IDC_CROSS));
}

void CGrabbedImageDialog::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	if (m_bCaptured) {
		::ReleaseCapture();
		m_bCaptured = FALSE;
	}
}

void CGrabbedImageDialog::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CRect rectBox;
	if (m_bCaptured) {
		DrawNewRect(&m_rectBox, point);
	}
}

#if 0
void CGrabbedImageDialog::OnPaint() 
{
	RedrawWindow(NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW);
}
#endif

void CGrabbedImageDialog::OnClose() 
{
	// TODO: Add your message handler code here and/or call default
	SetWindowPos(&wndTop,0,0,0,0, SWP_HIDEWINDOW| SWP_NOMOVE | SWP_NOSIZE);
	//	CImgWnd::OnClose();
}

CRect CGrabbedImageDialog::GetBoundingBox()
{
	return m_rectBox;
}
