/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// LafterDlg.cpp : implementation file
// Created	: 10 07 1997
//
// Nuria Oliver 1997, nuria@media.mit.edu


#include "stdafx.h"
#include "Lafter.h"
#include "LafterDlg.h"

#include "StaticFrameDialog.h"
#include "StaticFrameBBDialog.h"
#include "HistogramDialog.h"
#include "TrackerDialog.h"
#include "UErrors.h"
#include "MessageBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define WM_LAFTERQUIT WM_USER + 10
#define WM_CLOSEALLWINDOWS + 11

// BMINFO variables
static	BITMAPINFO_8	reducedBmInfo;
static	BITMAPINFO_8	histoBmInfo;

// Critical sections
static		CRITICAL_SECTION	g_csHisto;

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About
enum DisplayDialogs {
	NO_ID,
		LIVE,
		ESTIMATION,
		LEARNING,
		FILES,
		ADAPT,
		FACECONNECT,
		MOUTHCONNECT,
};

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();
	
	// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA
	
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL
	
	// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
//{{AFX_MSG_MAP(CAboutDlg)
// No message handlers
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLafterDlg dialog

CLafterDlg::CLafterDlg(CWnd* pParent /*=NULL*/)
: CDialog(CLafterDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLafterDlg)
	m_bEstimation = FALSE;
	m_bFaceHisto = FALSE;
	m_bLive = FALSE;
	m_bFaceConnect = FALSE;
	m_bTracker = FALSE;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	
	m_pLiveDlg = new CStaticFrameDialog(this, LIVE);
	m_pEstimationDlg  = new CStaticFrameBBDialog(this, ESTIMATION);
	m_pFaceConnectDlg  = new CStaticFrameDialog(this, FACECONNECT);
	m_pHistogramDlg = new CHistogramDialog(this);
	m_pTrackerDlg = new CTrackerDialog(this);	
	
	m_MinX = 0;
	m_MinY = 100;
	
	bCleanedUp = FALSE;
	m_bWindowsClosed = FALSE;
}

CLafterDlg::~CLafterDlg()
{	
	delete m_pLiveDlg;
	delete m_pEstimationDlg;
	delete m_pFaceConnectDlg;
	delete m_pHistogramDlg;
	delete m_pTrackerDlg;
}

void CLafterDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLafterDlg)
	DDX_Check(pDX, IDC_ESTIMATION, m_bEstimation);
	DDX_Check(pDX, IDC_FACEHISTO, m_bFaceHisto);
	DDX_Check(pDX, IDC_LIVE, m_bLive);
	DDX_Check(pDX, IDC_FACECONNECT, m_bFaceConnect);
	DDX_Check(pDX, IDC_TRACKER, m_bTracker);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CLafterDlg, CDialog)
//{{AFX_MSG_MAP(CLafterDlg)
ON_WM_SYSCOMMAND()
ON_WM_PAINT()
ON_WM_QUERYDRAGICON()
ON_BN_CLICKED(IDC_LIVE, OnLive)
ON_BN_CLICKED(IDC_FACEHISTO, OnFacehisto)
ON_BN_CLICKED(IDC_FACECONNECT, OnFaceconnect)
ON_BN_CLICKED(IDC_ESTIMATION, OnEstimation)
ON_BN_CLICKED(IDC_TRACKER, OnTracker)
ON_BN_CLICKED(IDC_QUIT, OnQuit)
ON_WM_TIMER()
ON_WM_CLOSE()
//}}AFX_MSG_MAP
ON_MESSAGE(WM_HGGOODBYE, OnHGGoodbye)
ON_MESSAGE(WM_TKGOODBYE, OnTKGoodbye)
ON_MESSAGE(WM_DDGOODBYE, OnDDGoodbye)
ON_MESSAGE(WM_LAFTERQUIT, OnQuit)
ON_MESSAGE(WM_CLOSEALLWINDOWS, GoodBye)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLafterDlg message handlers

BOOL CLafterDlg::OnInitDialog()
{
	ErrorNumber			err	= kErrNoErr;
	
	CDialog::OnInitDialog();
	
	// Add "About..." menu item to system menu.
	
	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);
	
	CMenu* pSysMenu = GetSystemMenu(FALSE);
	CString strAboutMenu;
	strAboutMenu.LoadString(IDS_ABOUTBOX);
	if (!strAboutMenu.IsEmpty())
	{
		pSysMenu->AppendMenu(MF_SEPARATOR);
		pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
	}
	
	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	this->SetWindowPos(&wndTop,0,0,0,0, SWP_SHOWWINDOW | SWP_NOSIZE);
	
	// Initialize the grabber
	// Use default resolution of 320x240
	
	if (!grabber.Init( )) {
#ifdef DEBUG
		LPVOID		lpMsgBuf;
		TRACE2 ("Impossible to create the 0 %d\n", GetLastError ());
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf, 0, NULL);
		
		// Display the string.
		TRACE1 ("%s\n", (const char *)lpMsgBuf);		
		// Free the buffer.
		LocalFree( lpMsgBuf );
#endif
		// Quit
		CDialog::OnClose();
		return TRUE;
	}
	
	int	pixelSize 	= grabber.PixelSize();
	int	width		= grabber.Width();
	int	height	= grabber.Height();
	
	// Create and hide all the dialogs
	if (m_pLiveDlg->Create( pixelSize, 
		width, 
		height ))
	{
		m_pLiveDlg->SetWindowPos(	&wndTop,m_MinX,m_MinY,
			width + 50, 
			height + 50, 
			SWP_HIDEWINDOW );
		// Center the StaticFrame
		m_pLiveDlg->SetWindowText("Live Video");
		m_pLiveDlg->CenterBitmap();
		// Start grabbing 
		
		if (grabber.StartGrab(GetSafeHwnd(), NULL, 
			(&(m_pLiveDlg->m_picLiveImg)), TRUE) ) 
		{
			grabber.Dispose();
			m_pLiveDlg->Quit();
			CDialog::OnClose();			
			return TRUE;
		}
	}
	else
	{
		MessageBox("Error on Live Window Creation");
	}
	
	int	factor	=	3;
	int reducedWidth = width/factor;
	int reducedHeight = height/factor;
	int histoPixelSize = pixelSize/4;
	
	// Initialize the other dialogs
	if (m_pEstimationDlg->Create(	pixelSize, 
								reducedWidth, 
								reducedHeight, TRUE, TRUE, TRUE ))
	{
		m_pEstimationDlg->SetWindowPos(&wndTop,m_MinX+175,m_MinY+610,
			reducedWidth + 50, 
			reducedHeight + 50, 
			SWP_HIDEWINDOW );
		m_pEstimationDlg->SetWindowText("Live Estimation");
		// Center the StaticFrame
		m_pEstimationDlg->CenterBitmap();
	}
	else
	{
		MessageBox("Error on Estimation Window Creation");
	}
	
	if (m_pFaceConnectDlg->Create(	pixelSize, 
								reducedWidth, 
								reducedHeight ))
	{
		m_pFaceConnectDlg->SetWindowPos(&wndTop,m_MinX,m_MinY+610,
			reducedWidth + 50, 
			reducedHeight + 50, 
			SWP_HIDEWINDOW );
		m_pFaceConnectDlg->SetWindowText("Face Connect");
		// Center the StaticFrame
		m_pFaceConnectDlg->CenterBitmap();
	}
	else
	{
		MessageBox("Error on FaceConnect Window Creation");
	}
	
	// Histogram
	
	int			nInitThreshold = 100;
	float		fInitMaxDist=50.0f;
	if (m_pHistogramDlg->Create(histoPixelSize, 
		reducedWidth,
		reducedHeight, &grabber,
		nInitThreshold, fInitMaxDist ))
	{
		m_pHistogramDlg->SetWindowPos(&wndTop,m_MinX+375,m_MinY,0,0,
			SWP_HIDEWINDOW | SWP_NOSIZE);
	}
	else
	{
		MessageBox("Error on Histogram Window Creation");
	}
	
				
	if (m_pTrackerDlg->Create()) {
		m_pTrackerDlg->SetWindowPos(&wndTop,m_MinX+375,m_MinY+300,0,0,SWP_HIDEWINDOW | SWP_NOSIZE);	
	}
	else {
		MessageBox("Error on Tracker Window Creation");
	}
				
				// Start vision
				
				if ( m_vision.Init( &grabber, factor ) != kErrNoErr ) {
					grabber.StopGrab();
					m_pLiveDlg->Quit();
					grabber.Dispose();
					// commit suicide
					CDialog::OnClose();
				}
				
				m_vision.StartVision(	m_pEstimationDlg, 
					m_pHistogramDlg,
					m_pTrackerDlg,
					m_pFaceConnectDlg);
				
				SetTimer(1,500,NULL);
				// Set the timer for the tracker
				
				m_pTrackerDlg->SetTimer( 1, 200, NULL );
				return TRUE;  // return TRUE  unless you set the focus to a control
}

void CLafterDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CLafterDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting
		
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
		
		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;
		
		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CLafterDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


//
// CLafterDlg user's defined functions
//
void CLafterDlg::OnLive() 
{
	// TODO: Add your control notification handler code here
	if(m_bLive) { 
		m_pLiveDlg->Hide();
		m_bLive = FALSE;
	} 
	else {
		m_pLiveDlg->Show();
		m_bLive = TRUE;
	}
}

void CLafterDlg::OnFacehisto() 
{
	// TODO: Add your control notification handler code here
	if(m_bFaceHisto) {
		m_pHistogramDlg->Hide();
		m_bFaceHisto = FALSE;
	} 
	else {
		m_pHistogramDlg->Show();
		m_bFaceHisto = TRUE;
	}
}

void CLafterDlg::OnFaceconnect() 
{
	// TODO: Add your control notification handler code here
	if (m_bFaceConnect) {
		m_pFaceConnectDlg->Hide();
		m_bFaceConnect = FALSE;
	} 
	else {
		m_pFaceConnectDlg->Show();
		m_bFaceConnect = TRUE;
	}
}

void CLafterDlg::OnEstimation() 
{
	// TODO: Add your control notification handler code here
	if(m_bEstimation) { 
		// hide the dialog
		m_pEstimationDlg->Hide();
		m_bEstimation = FALSE;
	} 
	else {
		// show the dialog
		m_pEstimationDlg->Show();
		m_bEstimation = TRUE;
	}
}

void CLafterDlg::GoodBye()
{
	OnHGGoodbye(0,0);
	OnDDGoodbye(LIVE,0);
	OnDDGoodbye(ESTIMATION,0);
	OnDDGoodbye(FACECONNECT,0);
	OnTKGoodbye(0,0);
}

void CLafterDlg::CleanUp()
{
	// TODO: Add your message handler code here and/or call default
	
	if (!bCleanedUp) {
		m_pTrackerDlg->KillTimer( 1 );
		KillTimer(1); 
		
		ASSERT( grabber.Grabbing() );
		grabber.StopGrab();
		
		while(grabber.Grabbing()) {
			Sleep(10);
		}
		
		m_vision.StopVision();
		
		while(m_vision.VisionOn()) {
			Sleep(10);
		}
		
		// Post destroy messages to all the dialogs
		
		m_vision.Dispose();
		grabber.Dispose();
		
		m_pLiveDlg->DestroyWindow();
		m_pEstimationDlg->DestroyWindow();
		m_pFaceConnectDlg->DestroyWindow();
		m_pHistogramDlg->DestroyWindow();
		m_pTrackerDlg->DestroyWindow();
		
		bCleanedUp = TRUE;
	}
}

void CLafterDlg::OnQuit() 
{
	if (m_bWindowsClosed) {
		CMessageBox* ClosingDlg = new CMessageBox(this);
		ClosingDlg->Create("Please wait while Lafter shuts down...");
		CleanUp();
		ClosingDlg->Destroy();
		delete ClosingDlg;		
		PostMessage(WM_CLOSE);
	}
	else {
		GoodBye();
		m_bWindowsClosed = TRUE;
		while ( !PostMessage(WM_LAFTERQUIT) );
	}
}

long CLafterDlg::OnHGGoodbye(UINT wParam, long lParam)
{
	m_bFaceHisto = FALSE;
	m_pHistogramDlg->Hide();
	CButton* pButton = (CButton*) GetDlgItem(IDC_FACEHISTO);
	pButton->SetCheck(0);
	
	return 0L;
}

long CLafterDlg::OnTKGoodbye(UINT wParam, long lParam)
{
	
	m_bTracker = FALSE;
	m_pTrackerDlg->Hide();
	CButton* pButton = (CButton*) GetDlgItem(IDC_TRACKER);
	pButton->SetCheck(0);
	
	return 0L;
}

long CLafterDlg::OnDDGoodbye(UINT wParam, long lParam)
{
	CButton* pButton;
	switch (wParam)
	{
	case NO_ID:
		break;
	case LIVE:
		m_bLive = FALSE;
		m_pLiveDlg->Hide();
		pButton = (CButton*) GetDlgItem(IDC_LIVE);
		pButton->SetCheck(0);
		break;
	case ESTIMATION:
		m_bEstimation = FALSE;
		m_pEstimationDlg->Hide();
		pButton = (CButton*) GetDlgItem(IDC_ESTIMATION);
		pButton->SetCheck(0);
		break;
	case FACECONNECT:
		m_bFaceConnect = FALSE;
		m_pFaceConnectDlg->Hide();
		pButton = (CButton*) GetDlgItem(IDC_FACECONNECT);
		pButton->SetCheck(0);
		break;
	}
	
	return 0L;
}


void CLafterDlg::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	switch (nIDEvent) {
	case 1:
		CString strText;
		strText.Format("%2.2f", grabber.FrameRate());
		SetDlgItemText(IDC_FRAMERATE, strText);
		break;
	}
	
	CDialog::OnTimer(nIDEvent);
}

void CLafterDlg::OnTracker() 
{
	// TODO: Add your control notification handler code here
	if(m_bTracker) { 
		m_pTrackerDlg->Hide();
		m_bTracker = FALSE;
	} 
	else {
		m_pTrackerDlg->Show();
		m_bTracker = TRUE;
	}	
}

void CLafterDlg::OnClose() 
{
	if (!bCleanedUp) {
		OnQuit();	
	}
	else {
		// commit suicide
		CDialog::OnClose();
	}
}
