/***************************************************************************
 *
 * Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
 * rights reserved.
 *
 * Developed  by the Perceptual Computing Section
 * at the Media Laboratory, MIT, Cambridge, Massachusetts.
 *
 * Permission to use, copy,  or modify this software and  its documentation
 * for  educational  and  research purposes only and without fee  is hereby
 * granted, provided  that this copyright notice and the original authors's
 * names appear  on all copies and supporting documentation.  If individual
 * files are  separated from  this  distribution directory  structure, this
 * copyright notice must be included.  For any other uses of this software,
 * in original or  modified form, including but not limited to distribution
 * in whole or in  part, specific  prior permission  must be  obtained from
 * MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
 *   basis  of  a commercial  software  or  hardware product  without first
 * obtaining appropriate licenses  from MIT.  MIT. makes no representations
 * about the suitability of this  software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 **************************************************************************/

///////////////////////////////////////////////////////////////////////////
// Filename	: CStaticFrameBB.cpp
// Created	: 07 07, 1997
// Description: A static frame class to display images 
//              with bounding box rectangle and eigen axes.
//
// by Nuria Oliver 1997, nuria@media.mit.edu 
//             Vasanth Philomin 1997, vasi@cs.umd.edu
///////////////////////////////////////////////////////////////////////////
// CStaticFrameBB.cpp : implementation file
//

#include "stdafx.h"
#include "StaticFrameBB.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CStaticFrameBB

CStaticFrameBB::CStaticFrameBB()
: CStaticFrame()
{
	m_bDrawBB = FALSE;
	m_bDrawEig = FALSE;
}

CStaticFrameBB::CStaticFrameBB(CDialog *pDlg) 
: CStaticFrame( pDlg )
{
	m_bDrawBB = FALSE;
	m_bDrawEig = FALSE;
}

BEGIN_MESSAGE_MAP(CStaticFrameBB, CStatic)
	//{{AFX_MSG_MAP(CStaticFrameBB)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CStaticFrameBB message handlers

void CStaticFrameBB::Create(BITMAPINFO_8 &bmInfo, 
								   BOOL bMatchScreen, 
								   BOOL bDrawBB, 
								   BOOL bDrawEig)
{
	// Call the parent's CStaticFrame::InitializeDib
	CStaticFrame::Create( bmInfo, bMatchScreen );
	
	// Flag to indicate whether to draw bounding box or not
	m_bDrawBB = bDrawBB;
	m_bDrawEig = bDrawEig;

	// Create default pen in blue and 2 pixels wide
	m_pen.CreatePen(PS_SOLID, 2, RGB(0,255,255));
	m_brush.CreateSolidBrush(RGB(255,0,0));
	if ( m_bDrawBB )
		m_rect.SetRect(0,0,0,0);
	if ( m_bDrawEig ) {
		for ( int i=0; i<4; i++ )
			m_arrEigPoint.Add( TPoint(0,0) );
	}
}

void CStaticFrameBB::OnPaint() 
{
	// Proceed only if Dib is valid
	if (m_bValidDib == NEVER)
		return;

	CPaintDC dc(this); // device context for painting
	
	m_dib.Draw(&dc, 0, 0);

	CPen *oldPen = dc.SelectObject(&m_pen);
	if (m_bDrawBB)	{
		CPen *oldPen = dc.SelectObject(&m_pen);
		CRgn rgnBox;

		dc.FrameRect(m_rect, &m_brush);
		dc.SelectObject(oldPen);
  }

	if (m_bDrawEig)	{
		CPen *oldPen = dc.SelectObject(&m_pen);
		// Draw the eigen lines (cross for Vasi)
		dc.MoveTo( m_arrEigPoint[0] );
		dc.LineTo( m_arrEigPoint[1] );
		dc.MoveTo( m_arrEigPoint[2] );
		dc.LineTo( m_arrEigPoint[3] );
		dc.SelectObject(oldPen);
	}

	if (m_bValidDib == ONCE)
		m_bValidDib = NEVER;

	ReleaseDC(&dc);
	// Do not call CFrameWnd::OnPaint() for painting messages
}
