/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// TrackerDialog.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CTrackerDialog dialog

#define WM_TKGOODBYE WM_USER + 700
#define WM_TKUPDATE WM_USER + 701
#define WM_UPDATE_DATA WM_USER + 702
#define WM_UPDATE_ERRORS WM_USER + 703

#include "UBoxTracker.h"

const	int		kTrackerDlgSliderMax	=	100;

class CTrackerDialog : public CDialog
{
	// Construction
public:
	float m_fHeightRatio;
	float m_fTheoHeightRatio;
	float m_fPError; 
	float m_fP; 
	int m_nD; 
	int m_fDError; 
	int m_nPanThreshold;
	int m_nTiltThreshold;
	float m_fHeightThreshold;
	float m_fTheoHeightThreshold;
	int m_nNumFrames;
	
	float m_fArrayHeightRatio[101];
	float m_fArrayTheoHeightRatio[101];
	float m_fArrayP[101];
	float m_fArrayHeightThreshold[101];
	float m_fArrayTheoHeightThreshold[101];
	
	float	m_fXErr; 
	float	m_fYErr; 
	float	m_fPanSpeed; 
	float	m_fTiltSpeed; 
	float	m_fZoomRatio; 
	BOOL  m_bPanic;
	
private:
	int m_nSliderHeightRatio;
	int m_nSliderTheoHeightRatio;
	int m_nSliderP; 
	int m_nSliderHeightThreshold; 
	int m_nSliderTheoHeightThreshold; 
	TBoxTracker				m_tracker;
	CRITICAL_SECTION			m_Sem;
	
	CDialog* m_pDlg;
public:
	CTrackerDialog(CWnd* pParent = NULL);   // standard constructor
	CTrackerDialog(CDialog* pDlg);
	~CTrackerDialog();
	
	BOOL Create();
	ErrorNumber Init();
	TBoxTracker*		BoxTracker() { return &m_tracker; }
	CRITICAL_SECTION*		Sem() { return &m_Sem;}
	void	SetParameters(  CRect*  bounds = (CRect*)NULL,
		float   heightRatio = -1,
		float   p = -1,
		float   d = -1,
		float   xThreshold = -1,
		float   yThreshold = -1,
		float   ratioThreshold = -1,
		float   theoRatio = -1,
		float   theoRatioThreshold = -1);
	
	void Hide() { SetWindowPos(&wndTop,0,0,0,0, SWP_HIDEWINDOW| SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER); }
	void Show()  { SetWindowPos(&wndTop,0,0,0,0, SWP_SHOWWINDOW| SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER); }
	void	Quit() { OnClose();}
	
	// Dialog Data
	//{{AFX_DATA(CTrackerDialog)
	enum { IDD = IDD_TRACKER };
	BOOL	m_bCamera;
	BOOL	m_bZoom;
	BOOL	m_bSaveParams;
	//}}AFX_DATA
	
	long OnUpdateData(UINT wParam, long lParam);
	long OnUpdateErrors(UINT wParam, long lParam);
	// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTrackerDialog)
public:
	virtual BOOL DestroyWindow();
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL
	
	
	// Implementation
protected:
	
	// Generated message map functions
	//{{AFX_MSG(CTrackerDialog)
	afx_msg void OnClose();
	afx_msg void OnCamera();
	afx_msg void OnZoom();
	afx_msg void OnSaveparams();
	virtual BOOL OnInitDialog();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
	
	DECLARE_MESSAGE_MAP()
		
private:
	void CleanUp();
};
