/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

//
//       File : UCDeviceComm.cpp
//       Goal : Generic serial communication setting
//
//		 Date : 02 07 97
//
// 	Nuria Oliver 1997, nuria@media.mit.edu
//				Vasanth Philomin 1997, vasi@cs.umd.edu
//

#include "stdAfx.h"
#include <afxmt.h>
#include "UCDeviceComm.h"

static CEvent event(FALSE, TRUE);	
// Used for synchronizing with the asynchronous read and write functions

CDeviceComm::CDeviceComm (void)
// The constructor
{
    m_hPort = INVALID_HANDLE_VALUE;		// indicating that no port is open
	
} // constructor


CDeviceComm::~CDeviceComm (void)
// The destructor
{
	Close ();	// close the port if it's in use.
	
} // destructor



//////////////////////////////////////////////////////////////////////////////
// Open
//
// Calls Windows API routine with standard LTI device operating characterics.
// Returns TRUE for sucess, else FALSE.
//////////////////////////////////////////////////////////////////////////////
BOOL CDeviceComm::Open (char *PortName, int Speed) {
	
	m_hPort = CreateFile (PortName, GENERIC_READ | GENERIC_WRITE,
		
		0,                    // exclusive access
		
		NULL,                 // no security attrs
		
		OPEN_EXISTING,
		
		//FILE_ATTRIBUTE_NORMAL, 
		
		FILE_FLAG_OVERLAPPED,  // overlapped I/O	
		
		NULL );
	
	if (m_hPort == INVALID_HANDLE_VALUE) {
		TRACE0 ("Unable to open COM port.\n");
		return	FALSE;
	}
	
#if 0
    SetupComm (m_hPort, 1024, 1024);
	
    SetCommMask (m_hPort, 0);
#endif
    PurgeComm (m_hPort, PURGE_TXABORT | PURGE_RXABORT |
		PURGE_TXCLEAR | PURGE_RXCLEAR);
	
#if 0
	COMMTIMEOUTS  CommTimeOuts;
	
	CommTimeOuts.ReadIntervalTimeout = MAXDWORD;
    CommTimeOuts.ReadTotalTimeoutMultiplier = 0;
    CommTimeOuts.ReadTotalTimeoutConstant = 0;
    CommTimeOuts.WriteTotalTimeoutMultiplier = 0;
    CommTimeOuts.WriteTotalTimeoutConstant = 0;
    SetCommTimeouts (m_hPort, &CommTimeOuts);
	
	DCB dcb;
	dcb.DCBlength = sizeof( DCB ) ;
    if (GetCommState (m_hPort, &dcb) < 0)  {
        // error getting comm state
        Close (); // close it since open succeeded
        return FALSE;
    }
	
    dcb.BaudRate = Speed;
    dcb.ByteSize = 8;
    dcb.Parity = 0;
    dcb.StopBits = ONESTOPBIT;
    dcb.fBinary = TRUE;        // don't do EOF detection, pass all
    dcb.fRtsControl = RTS_CONTROL_DISABLE;
    dcb.fParity = FALSE;
    dcb.fOutxCtsFlow = FALSE;  // no hardware flow control
    dcb.fOutxDsrFlow = FALSE;  // ditto
    dcb.fDtrControl = DTR_CONTROL_DISABLE;
	dcb.fDsrSensitivity = FALSE;
    dcb.fOutX = FALSE;         // no software flow control either
    dcb.fInX = FALSE;          // ditto
    dcb.fNull = FALSE;         // keep null characters
    dcb.XonChar = '\x0';       // the remaining fields are not used...
    dcb.XoffChar = '\x0';      // ...so their value is a "don't care"
    dcb.XonLim = 0;
    dcb.XoffLim = 0;
    dcb.ErrorChar = '\x0';
    dcb.EofChar = '\x0';
    dcb.EvtChar = '\x0';
	dcb.fAbortOnError = FALSE;
	
    if (SetCommState (m_hPort, &dcb) < 0) {
        // error setting comm state
        Close (); // close it since open succeeded
        return FALSE;
    }
#endif
	return TRUE;
} // Open



//////////////////////////////////////////////////////////////////////////////
// Close
//
// Call this when done with a port opened via Open method above.
//////////////////////////////////////////////////////////////////////////////
void CDeviceComm::Close (void)
{
    if (m_hPort == INVALID_HANDLE_VALUE)
        // port not open, so just leave
        return;
	
	Purge();
    int RetVal = CloseHandle (m_hPort);
    ASSERT (RetVal == TRUE); // should succeed
	
    m_hPort = INVALID_HANDLE_VALUE;   // to show that no longer associated with a valid port
} // Close



//////////////////////////////////////////////////////////////////////////////
// GetChar
//
// Reads a character from the port, if available. Returns FALSE if none
// available. Return TRUE and sets reference passed to the character read
// if one was available.
//////////////////////////////////////////////////////////////////////////////
BOOL CDeviceComm::GetChar (char &ch)
{
	//	return FALSE;
	
    DWORD ReadLeng;
	BOOL ReadRes = ReadFile (m_hPort, &ch, 1, &ReadLeng, NULL);
    if (ReadLeng > 0)
        return TRUE;
	
	if (GetLastError () == ERROR_HANDLE_EOF)
		return FALSE;
	
	// Erroris not because characters are not available, could
	// further decode here.
	
	return FALSE;
	
}

//////////////////////////////////////////////////////////////////////////////
// PutChar for a single char
//
// Writes the character passed to thte port.
//////////////////////////////////////////////////////////////////////////////
void CDeviceComm::PutChar (char ch)
{
    DWORD WriteLeng;
	if (WriteFile (m_hPort, &ch, 1, &WriteLeng, NULL) == 0) {
        // An error ocurred.
        TRACE2 ("Error writing port %d. Error = %lu\n", m_hPort, GetLastError ());
    }
} // PutChar



//////////////////////////////////////////////////////////////////////////////
// PutChar for a character string
//
// Writes the character string passed to the port.
//////////////////////////////////////////////////////////////////////////////
ErrorNumber CDeviceComm::PutBuffer (char *buf, int length)
{
	DWORD nBytesWritten;	// Number of bytes written
	
	OVERLAPPED Overlapped;
	Overlapped.Internal = 0; 
    Overlapped.InternalHigh = 0; 
    Overlapped.Offset = 0; 
    Overlapped.OffsetHigh = 0; 
	Overlapped.hEvent = event.m_hObject;	// Signal an event kernel object
	ErrorNumber err = kErrNoErr;
	
	// Start writing the data asynchronously
	if (!WriteFile(m_hPort, buf, length, &nBytesWritten, &Overlapped))	{
		if (GetLastError() == ERROR_IO_PENDING)	{
			// Note that WritFile will return FALSE since it is asynchronous and all of the data hasnt been
			// written yet. Also, nBytesWritten is not going to contain the number of bytes written ultimately.
			
			// Wait forever the device kernel object to be signalled
			::WaitForSingleObject(event.m_hObject, INFINITE);
			
			event.ResetEvent();
			// Check how much data was written - GetOverlappedResult will return TRUE since IO has completed
			//	if (!GetOverlappedResult(m_hPort, &Overlapped, &nBytesWritten, FALSE))
			//		return FALSE;
			if (!GetOverlappedResult(m_hPort, &Overlapped, &nBytesWritten, FALSE))
				err = kErrIOPending;
		}
		else
			err = kErrIOErr;
	}
	
	
#ifdef DEBUG
	LPVOID		lpMsgBuf;
    TRACE2 ("Writing port %d. Error = %lu\n", m_hPort, GetLastError ());
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf, 0, NULL);
	
	// Display the string.
	TRACE1 ("%s\n", (const char *)lpMsgBuf);
	
	// Free the buffer.
	LocalFree( lpMsgBuf );
	
	TRACE1 ("Number of bytes written = %d\n", nBytesWritten);
#endif
	
	return err;
} // PutBuffer

BOOL CDeviceComm::Purge()
{
	return PurgeComm (m_hPort, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
	//return FlushFileBuffers( m_hPort );
}

ErrorNumber CDeviceComm::GetBuffer(char *buf, int length, DWORD *nRead, DWORD timeout)
{
	
	OVERLAPPED Overlapped;
	//memset(&Overlapped, 0, sizeof(Overlapped));
	Overlapped.Internal = 0; 
    Overlapped.InternalHigh = 0; 
    Overlapped.Offset = 0; 
    Overlapped.OffsetHigh = 0; 
	Overlapped.hEvent = event.m_hObject;	// Signal an event kernel object
	
	// Start reading the data asynchronously
	if (!ReadFile(m_hPort, buf, length, nRead, &Overlapped))	{
		if (GetLastError() == ERROR_IO_PENDING)	{
			// Note that WritFile will return FALSE since it is asynchronous and all of the data hasnt been
			// written yet. Also, nBytesWritten is not going to contain the number of bytes written ultimately.
			
			// Wait for the device kernel object to be signalled
			if (::WaitForSingleObject(event.m_hObject, timeout) == WAIT_TIMEOUT)	{
				// Check how much data was written
				GetOverlappedResult(m_hPort, &Overlapped, nRead, FALSE);
				return kErrTimeOut;
			}
			else
				event.ResetEvent();
			
			// Check how much data was written - GetOverlappedResult will return TRUE since IO has completed
			if (!GetOverlappedResult(m_hPort, &Overlapped, nRead, FALSE))
				return kErrIOPending;
		}
		else
			return kErrIOErr;
	}
	
#ifdef DEBUG
	LPVOID		lpMsgBuf;
	
    TRACE2 ("Reading port %d. Error = %lu\n", m_hPort, GetLastError ());
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf, 0, NULL);
	
	// Display the string.
	TRACE1 ("%s\n", (const char *)lpMsgBuf);
	
	// Free the buffer.
	LocalFree( lpMsgBuf );
	
	//BOOL ReadRes = ReadFile (m_hPort, buf, length, nRead, NULL);
	TRACE1 ("Number of bytes read = %d\n", nBytesRead);
	TRACE1 ("%s\n", (const char *)buf);
#endif
	
	return kErrNoErr;
}

