/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// File       : UCamControl.cc
// Creation   : 27 08 1996
// Goal       : defines TCamControl virtual class. See "UCamControl.h"
//
//  Nuria Oliver 1997, nuria@media.mit.edu


// unit include
#include "stdAfx.h"
#include <afxmt.h>
#include "UCamControl.h"


// project includes
#include "UObjAlloc.h"

//
//    Init
//

ErrorNumber TCamControl::Init (char *device)
{
	ErrorNumber   err = kErrNoErr;
	
	// init data members
	m_bReset          = TRUE;
	m_bHome           = FALSE;
	m_fPanPos         = 0.0F;
	m_fTiltPos        = 0.0F;
	m_fPanSpeed       = 0.0F;
	m_fTiltSpeed      = 0.0F;
	m_nZoomSpeed      = 0;
	m_fZoomPos		= 0.0F;
	
	m_nFSMState		= 0;
	m_nParentID       = 0;
	
	::InitializeCriticalSection(&m_pSem);
	
	stopEvent.ResetEvent();
	m_bChildDead = FALSE;
	AfxBeginThread(CamControlChildMain, (LPVOID)this, THREAD_PRIORITY_NORMAL);
	
	return err;
}




//
//    Dispose
//

void TCamControl::Dispose ()
{
	stopEvent.SetEvent();
	
	while (!m_bChildDead) {
		Sleep(10);
	}
	
	::DeleteCriticalSection(&m_pSem);
}




//
//    LowLevelProcess
//

void TCamControl::LowLevelProcess ()
{
	BOOL lReset, lHome;	// Local copies
	::EnterCriticalSection( &m_pSem );
	lReset = m_bReset;
	lHome = m_bHome;
	::LeaveCriticalSection( &m_pSem );
	
	if (lReset)
	{
		this->DoReset ();
		::EnterCriticalSection( &m_pSem );
		m_bReset = FALSE;
		::LeaveCriticalSection( &m_pSem );
	}
	else
		if (lHome)
		{
			this->DoHome ();
			::EnterCriticalSection( &m_pSem );
			m_bHome = FALSE;
			::LeaveCriticalSection( &m_pSem );
		}
		else
		{
			switch (m_nFSMState)
			{
			/*
			case 0:
			// ask for absolut Pan position
			this->UpdatePanPos ();
			break;
			
			  case 3:
			  // ask for absolut Tilt position
			  this->UpdateTiltPos ();
			  break;
			  
				*/
			case 0:
				// ask for absolut Zoom position
				this->UpdateZoomPos ();
				Sleep(40);
				break; 
			case 2:
				this->ProcessSpeedCommand ();
				break;
			case 1:	
				this->ProcessZoomCommand ();
				//Sleep(80);
				break;
				
			}
			m_nFSMState++;
			m_nFSMState %= 3;
		}
}




//
//    CamControlChildMain
//

UINT CamControlChildMain (LPVOID userData)
{
	TCamControl*            cam = (TCamControl*)userData;
	
	while (TRUE)
	{
		if (::WaitForSingleObject(cam->stopEvent.m_hObject, 0) == WAIT_OBJECT_0)
			break;
		cam->LowLevelProcess ();
		Sleep(0);
	}
	
	cam->GoHome();
	Sleep(200);
	cam->m_bChildDead = TRUE;
	return 0;
}



void TCamControl::Stop()
{
	this->DoStop();
}
