/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// File       : UEVID30.cc
// Creation   : Aug 21 1996
// Goal       : TEVID30 class definition. See "UEVID30.h".
//
//  Nuria Oliver 1997, nuria@media.mit.edu

#include "stdafx.h"
#include "Winbase.h"
// unit include

// project includes
#include "UObjAlloc.h"
#include "UTimer.h"
#include "UEVID30.h"

// ANSI includes
#include <math.h>
#include <iostream.h>
#include <fstream.h>


// warning, the 2 folowing constants have been determined by a very
// inaccurate method

const float   kPanSpeedConvert    = 0.33F;
// ratio pan camera units over deg/s

const float   kTiltSpeedConvert   = 0.42F;
// ratio tilt camera units over deg/s


const UINT  kEVIBigTimeOut      = 7000;
// rs232 timeout

//
//    Init
//
ErrorNumber TEVID30::Init (char* device)
{
	ErrorNumber       err = kErrNoErr;
	// struct termios    tdata;
	//int               flags;
	
	// init data members
	m_pTimer    = (TTimer*)NULL;
	m_nToRead   = 0;
	m_nZooming  = 0;
	
	// open the serial port as a file with the correct
	// parameters: see UCDeviceComm for details
	if (!m_SerialPort.Open(device, 9600))
		return( kErrIOErr );
	
	if (err == kErrNoErr)
		err = AllocAndInit (m_pTimer);
    // set some stuff
    //tdata.c_iflag     = 0;
    //tdata.c_oflag     = 0;
    //tdata.c_lflag     = 0;
    //tdata.c_cc[VMIN]  = 0;
    //tdata.c_cc[VTIME] = 3;
	
    // set serial port to 9600 bauds / 8 bit data / local / read enabled
    //tdata.c_cflag = B9600 | CS8 | CLOCAL | CREAD;
	
    // setup serial line parameters
    //if (ioctl (fFD, TCSETS, &tdata) < 0)
	// allocate and initialise the timer object
	
	if ( err != kErrNoErr ) {
		m_SerialPort.Close();
		return err;
	}
	
	memset( m_cBuffer, 0, sizeof(m_cBuffer) );
	// clear input buffer
	this->ClearBuffer ();
	
	// call inherited::Init
	err = TCamControl::Init (device);
	
	// error handling
	if (err != kErrNoErr)
		this->Dispose ();
	
	return err;
}




//
//    Dispose
//

void TEVID30::Dispose ()
{
	
	// call inherited::Dispose
	TCamControl::Dispose ();
	
	// Dispose the timer
	TestDisposeAndFree (m_pTimer);
	
	// close the serial port
	//  if (fFD != 0)
	//  {
	//    close(fFD);
	//    fFD = 0;
	//  }
	m_SerialPort.Close();
}




//
//    AskMaxPanSpeed
//

ErrorNumber TEVID30::AskMaxPanSpeed (float& max)
{
	ErrorNumber err = kErrNoErr;
	
	m_cBuffer[0] = (char)0x81;
	m_cBuffer[1] = (char)0x09;
	m_cBuffer[2] = (char)0x06;
	m_cBuffer[3] = (char)0x11;
	m_cBuffer[4] = (char)0xFF;
	
	err = this->SendAndReceive (5, 5);
	
	if (err == kErrNoErr)
		max = ((float)m_cRecBuffer[2]) / kPanSpeedConvert;
	
	return err;
}




//
//    AskMaxTiltSpeed
//

ErrorNumber TEVID30::AskMaxTiltSpeed (float& max)
{
	ErrorNumber err = kErrNoErr;
	
	m_cBuffer[0] = (char) 0x81;
	m_cBuffer[1] = (char) 0x09;
	m_cBuffer[2] = (char) 0x06;
	m_cBuffer[3] = (char) 0x11;
	m_cBuffer[4] = (char) 0xFF;
	
	err = this->SendAndReceive (5, 5);
	
	if (err == kErrNoErr)
		max = ((float)m_cRecBuffer[3]) / kTiltSpeedConvert;
	
	return err;
}




//
//    DoReset
//

void TEVID30::DoReset ()
{
	this->DoStop ();
	
	m_cBuffer[0] = (char) 0x81;
	m_cBuffer[1] = (char) 0x01;
	m_cBuffer[2] = (char) 0x06;
	m_cBuffer[3] = (char) 0x05;
	m_cBuffer[4] = (char) 0xFF;
	
	this->SendAndReceive (5, 3);
	
}




//
//    DoHome
//

void TEVID30::DoHome ()
{
	this->DoStop ();
	
	m_cBuffer[0] = (char) 0x81;
	m_cBuffer[1] = (char) 0x01;
	m_cBuffer[2] = (char) 0x06;
	m_cBuffer[3] = (char) 0x04;
	m_cBuffer[4] = (char) 0xFF;
	Sleep(50);
	this->SendAndReceive (5, 3, kEVIBigTimeOut);
}




//
//    DoStop
//

void TEVID30::DoStop ()
{
	m_cBuffer[0] = (char) 0x81;
	m_cBuffer[1] = (char) 0x01;
	m_cBuffer[2] = (char) 0x06;
	m_cBuffer[3] = (char) 0x01;
	m_cBuffer[4] = (char) 0x09;
	m_cBuffer[5] = (char) 0x00;
	m_cBuffer[6] = (char) 0x03;
	m_cBuffer[7] = (char) 0x03;
	m_cBuffer[8] = (char) 0xff;
	
	this->SendAndReceive (9, 6, kEVISmallTimeOut);
}




//
//    ProcessZoomCommand
//

void TEVID30::ProcessZoomCommand ()
{
	//m_pSem->SemP ();
	::EnterCriticalSection( &m_pSem );
	int           theZoom       = m_nZoomSpeed;
	::LeaveCriticalSection( &m_pSem );
	//m_pSem->SemV ();
	
	if (m_nZooming != theZoom)
	{
		
		m_cBuffer[0] = (char) 0x81;
		m_cBuffer[1] = (char) 0x01;
		m_cBuffer[2] = (char) 0x04;
		m_cBuffer[3] = (char) 0x07;
		
		m_cBuffer[5] = (char) 0xff;
		
		switch (theZoom)
		{
		case 2:
			m_cBuffer[4] = 0x27;
			break;
			
		case 1:
			m_cBuffer[4] = 0x02;
			break;
			
		case 0:
			m_cBuffer[4] = 0x00;
			break;
			
		case -2:
			m_cBuffer[4] = 0x37;
			break;
			
		case -1:
			m_cBuffer[4] = 0x03;
			break;
		}
		
		//	Sleep(50);
		this->SendAndReceive (6, 6, kEVISmallTimeOut);
		if (m_nToRead == 0)
			m_nZooming = theZoom;
	}
}




//
//    ProcessSpeedCommand
//

void TEVID30::ProcessSpeedCommand ()
{
	unsigned char   pan, tilt;
	float           thePan, theTilt;
	
	//m_pSem->SemP ();
	::EnterCriticalSection( &m_pSem );
	thePan  = m_fPanSpeed;
	theTilt = m_fTiltSpeed;
	::LeaveCriticalSection( &m_pSem );
	//m_pSem->SemV ();
	
	pan   = (UCHAR)(fabs (thePan)  * kPanSpeedConvert);
	tilt  = (UCHAR)(fabs (theTilt) * kTiltSpeedConvert);
	
	if (pan > 0x18)
		pan = 0x18;
	
	if (tilt > 0x14)
		tilt = 0x14;
	
	m_cBuffer[0] = (char) 0x81;
	m_cBuffer[1] = (char) 0x01;
	m_cBuffer[2] = (char) 0x06;
	m_cBuffer[3] = (char) 0x01;
	m_cBuffer[4] = (char) pan;
	m_cBuffer[5] = (char) tilt;
	
	m_cBuffer[8] = (char) 0xff;
	
	if (thePan == 0)
	{
		if (theTilt == 0)
		{
			m_cBuffer[6] = 0x03;
			m_cBuffer[7] = 0x03;
		}
		else
			if (theTilt > 0)
			{
				m_cBuffer[6] = 0x03;
				m_cBuffer[7] = 0x01;
			}
			else
			{
				m_cBuffer[6] = 0x03;
				m_cBuffer[7] = 0x02;
			}
	}
	else
		if (thePan > 0)
		{
			if (theTilt == 0)
			{
				m_cBuffer[6] = 0x01;
				m_cBuffer[7] = 0x03;
			}
			else
				if (theTilt > 0)
				{
					m_cBuffer[6] = 0x01;
					m_cBuffer[7] = 0x01;
				}
				else
				{
					m_cBuffer[6] = 0x01;
					m_cBuffer[7] = 0x02;
				}
		}
		else
		{
			if (theTilt == 0)
			{
				m_cBuffer[6] = 0x02;
				m_cBuffer[7] = 0x03;
			}
			else
				if (theTilt > 0)
				{
					m_cBuffer[6] = 0x02;
					m_cBuffer[7] = 0x01;
				}
				else
				{
					m_cBuffer[6] = 0x02;
					m_cBuffer[7] = 0x02;
				}
		}
		
		//  Sleep(20);
		this->SendAndReceive (9, 6, kEVISmallTimeOut);
}




//
//    UpdatePanPos
//

void TEVID30::UpdatePanPos ()
{
	m_fPanPos = 0;
}




//
//    UpdateTiltPos
//

void TEVID30::UpdateTiltPos ()
{
	m_fTiltPos = 0;
}



//
//    UpdateZoomPos
//

void TEVID30::UpdateZoomPos ()
{
	ErrorNumber   err;
	float         x, x2, x3;
	
	m_cBuffer[0] = (char) 0x81;
	m_cBuffer[1] = (char) 0x09;
	m_cBuffer[2] = (char) 0x04;
	m_cBuffer[3] = (char) 0x47;
	m_cBuffer[4] = (char) 0xFF;
	
	err = this->SendAndReceive (5, 7, kEVISmallTimeOut);
	
	if (err == kErrNoErr)
	{
		x = (float) m_cRecBuffer[3] * 256 + m_cRecBuffer[4] * 16 + m_cRecBuffer[5];
		
		x2 = x*x;
		x3 = x2*x;
		
		m_fZoomPos = (float) (0.0000005 * x3 -0.00017 * x2 + 0.09 * x + 46);
	}
}



//
//    SendAndReceive
//

inline
ErrorNumber TEVID30::SendAndReceive (int out, int in, UINT time)
{
	ErrorNumber   err = kErrNoErr;
	
	do	{
		ClearBuffer();
		err = this->SendBuffer (out);
		
		if (err == kErrNoErr)	{
			memset(m_cRecBuffer, 0, sizeof(m_cRecBuffer));
			err = this->ReceiveBuffer (in, time);
		}
	}
	while (err == kErrCameraBusy);
	
	return err;
}




//
//    ClearSendAndReceive
//

inline
ErrorNumber TEVID30::ClearSendAndReceive (int out, int in, UINT time)
{
	ErrorNumber   err;
	
	this->ClearBuffer ();
	
	err = this->SendBuffer (out);
	
	if (err == kErrNoErr)
		err = this->ReceiveBuffer (in, time);
	
	return err;
}




//
//    SendBuffer
//

ErrorNumber TEVID30::SendBuffer (int length)
{
	ErrorNumber   err   = kErrNoErr;
	ULONG         nWr   = 0; 
	char*         src   = m_cBuffer;
	
	
	err = m_SerialPort.PutBuffer(src, length);
	
#if 0
	LPVOID		lpMsgBuf;
	if (err != kErrNoErr)	{
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf, 0, NULL);
		// Display the string.
		MessageBox( NULL, (const char *)lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );
		
		// Free the buffer.
		LocalFree( lpMsgBuf );
	}
#endif
	
	return err;
}




//
//    ReceiveBuffer
//

ErrorNumber TEVID30::ReceiveBuffer (int length, UINT time)
{
	ErrorNumber   err		= kErrNoErr;
	DWORD         nRead		= 0;
	DWORD		  nRead1;
	char		 *ptr		= m_cRecBuffer;
	
	if (m_nToRead != 0)	{
		err = m_SerialPort.GetBuffer(ptr, m_nToRead, &nRead, time);
		m_nToRead -= nRead;
	}
	
	if (m_nToRead > 0)	{
		m_nToRead += length;
		ASSERT(m_nToRead + length <= kEVICmdBufferSize);
	}
	else	{	// m_nToRead is 0
		nRead1 = 0;
		ptr += nRead;
		err = m_SerialPort.GetBuffer(ptr, length, &nRead1, time);
		if (nRead1 >= 3)	{
			if (ptr[2] != (char)0xFF)	{
				err = kErrCameraBusy;
				return err;
			}
		}
		if ( err == kErrTimeOut )
			m_nToRead = length-nRead1;
	}
	
	//err = m_SerialPort.GetBuffer(m_cRecBuffer, m_nToRead+length, &nRead, time);
	
	return err;
}




//
//    ClearBuffer
//

void TEVID30::ClearBuffer ()
{
	//char          buffer[10];
	
	//while (read (fFD, &buffer, 10) > 0);
	// Flush the device
	m_SerialPort.Purge();
}