/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// File       : UEVID30.h
// Creation   : Aug 21 1996
// Goal       : low level driver for SONY EVI-D30 D31 pan/tilt/zoom camera.
//
//  Nuria Oliver 1997, nuria@media.mit.edu

#ifndef _UEVID30_H
#define _UEVID30_H

// project includes
#include "UCamControl.h"
#include "UErrors.h"
#include "UCDeviceComm.h"

static const int    kEVICmdBufferSize = 128;   // command buffer size (bytes)

class TEVID30 : public TCamControl
{
public:
	int GetFZooming()	{ return m_nZooming; }
	
	ErrorNumber     Init (char* device = "COM1");
    // open connection to the camera using the serial port accessed by
    // the device faile name <deviceName>
	
    void            Dispose ();
    // close connection to the camera
	
    ErrorNumber     AskMaxPanSpeed (float& max);
    // ask the camera for maximum pan speed (deg/sec)
	
    ErrorNumber     AskMaxTiltSpeed (float& max);
    // ask the camera for maximum tilt speed (deg/sec)
	
	
protected:
	char			m_cRecBuffer[kEVICmdBufferSize];	// Buffer for receiving values from camera
	CDeviceComm		m_SerialPort;
	
    char            m_cBuffer[kEVICmdBufferSize];
	// buffer for sending commands
    TTimer*         m_pTimer;         // timer for detecting timout
    int             m_nToRead;        // number of chars that could not be read
    int             m_nZooming;       // Camera zoom current status
	
    void            DoReset ();
    void            DoHome ();
    void            DoStop ();
    void            ProcessZoomCommand ();
    void            ProcessSpeedCommand ();
    void            UpdatePanPos ();
    void            UpdateTiltPos ();
    void            UpdateZoomPos ();
	
	
    inline
		ErrorNumber     ClearSendAndReceive (int out, int in, UINT time);
    // clear incoming data, send command and wait for reply
	
    inline
		ErrorNumber     SendAndReceive (int out, int in, UINT time = kEVISmallTimeOut);
    // send command and wait for reply
	
    ErrorNumber     SendBuffer (int length);
    // send the command buffer to the camera
	
    ErrorNumber     ReceiveBuffer (int length, UINT time);
    // get info from the camera
	
    void            ClearBuffer ();
    // flush data sent from the camera
};



#endif
