/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

// UMatrix4.h
// Nuria Oliver 1997, nuria@media.mit.edu

#ifndef _UMATRIX4_H_
#define _UMATRIX4_H_

//Class for symmetrical matrixes
template<class T> class TVector2;

template< class T >
class TMatrix4
{
    friend class TVector2<T>;
private:
    T *m_pAllocArray;
public:
    TMatrix4(){m_pAllocArray = new T[4];}
    TMatrix4(T e1, T e2, T e3, T e4){m_pAllocArray = new T[4];set(e1, e2, e3, e4);}
    TMatrix4( T* data ){m_pAllocArray = new T[4];set( data ); }
    TMatrix4( TMatrix4<T>&m ){m_pAllocArray = new T[4];set( m ); }
	
    ~TMatrix4() {if (m_pAllocArray) delete m_pAllocArray;}
	
	T* getData() { return m_pAllocArray; }
	
	T& operator()( int i ){return m_pAllocArray[i];}
	void setElem( int i, T val ){ m_pAllocArray[i]=val;}
	void set(T* v);
	void set(T e1, T e2, T e3, T e4);
	void set(TMatrix4<T>& m){set(m(0), m(1), m(2), m(3));}
	void setValue(T val);
	void setZero() {setValue(T(0));}
	void setIdentity();
	void setAddEq(T e1, T e2, T e3, T e4);
	void setAddEq(T* v1 );
	void setAdd(T* v1, T* v2 );
	void setSubtractEq(T e1, T e2, T e3, T e4);
	void setSubtractEq(T* v1 );
	void setSubtract(T* v1, T* v2 );
	void setOuterProduct( T* v );
	void setAddOuterProduct( T* v );
	void setSubtractOuterProduct( T* v );
	void setSubtractOuterProduct( TVector2<T>& v );
	void setOuterProduct( T e1, T e2 );
	void setOuterProduct( TVector2<T>& v ){ setOuterProduct( v(0), v(1) ); }
	void setOuterProduct( TVector2<T>& v1, TVector2<T>& v2 );
	void setAddOuterProduct( T e1, T e2 );
	void setAddOuterProduct( TVector2<T>& v ){setAddOuterProduct(v(0), v(1));}
	void setSubtractOuterProduct( T e1, T e2 );
	void setScaleEq( T val );
	void setDivideEq( T val );
	void setStringValue( char* s );
	
	int isSingular() { return ( getDet() == 0 ); }
    
	T getDet();
	int setInvert(TMatrix4<T>&);
	int setInvertEq();
    
	void operator*=(  T val ) { setScaleEq( val ); }
	void operator/=(  T val ) { setDivideEq( val ); }
	TMatrix4& operator=(  TMatrix4<T>& val ) { set( val ); return *this; }
	
    int operator==( const TMatrix4<T>& );
};

template<class T>
void TMatrix4<T>::set( T* v )
{
    for ( int i=0; i<4; i++ )
		m_pAllocArray[i]=v[i];
}

template<class T>
void TMatrix4<T>::set( T e1, T e2, T e3, T e4 )
{
    m_pAllocArray[0]=e1;
    m_pAllocArray[1]=e2;
    m_pAllocArray[2]=e3;
    m_pAllocArray[3]=e4;
}

template<class T> 
void TMatrix4<T>:: setValue( T val )
{
    set( val, val, val, val );
}

template<class T>
void TMatrix4<T>:: setIdentity()
{
    m_pAllocArray[0]=m_pAllocArray[3]=1;
    m_pAllocArray[1]=m_pAllocArray[2]=0;
}

template<class T>
void TMatrix4<T>:: setAddEq( T* v )
{
    for ( int i=0; i<4; i++ )
		m_pAllocArray[i]+=v[i];
}

template<class T>
void TMatrix4<T>:: setAddEq( T e1, T e2, T e3, T e4 )
{
    m_pAllocArray[0]+=e1;
    m_pAllocArray[1]+=e2;
    m_pAllocArray[2]+=e3;
    m_pAllocArray[3]+=e4;
}

template<class T>
void TMatrix4<T>:: setAdd( T* v1, T* v2 )
{
    m_pAllocArray[0]=v1[0]+v2[0];
    m_pAllocArray[1]=v1[1]+v2[1];
    m_pAllocArray[2]=v1[2]+v2[2];
    m_pAllocArray[3]=v1[3]+v[3];
}


template<class T>
void TMatrix4<T>:: setSubtractEq( T* v )
{
    m_pAllocArray[0]-=v[0];
    m_pAllocArray[1]-=v[1];
    m_pAllocArray[2]-=v[2];
    m_pAllocArray[3]-=v[3];
}

template<class T>
void TMatrix4<T>:: setSubtractEq( T e1, T e2, T e3, T e4 )
{
    m_pAllocArray[0]-=e1;
    m_pAllocArray[1]-=e2;
    m_pAllocArray[2]-=e3;
    m_pAllocArray[3]-=e4;
}

template<class T>
void TMatrix4<T>:: setSubtract( T* v1, T* v2 )
{
    m_pAllocArray[0]=v1[0]-v2[0];
    m_pAllocArray[1]=v1[1]-v2[1];
    m_pAllocArray[2]=v1[2]-v2[2];
    m_pAllocArray[3]=v1[3]-v2[3];
}

template<class T> 
void TMatrix4<T>:: setOuterProduct( T* v )
{
    m_pAllocArray[0]=v[0]*v[0];
    m_pAllocArray[1]=m_pAllocArray[2]=v[0]*v[1];
    m_pAllocArray[3]=v[1]*v[1];
}

template<class T> 
void TMatrix4<T>:: setOuterProduct( T e1, T e2 )
{
    m_pAllocArray[0]=e1*e1;
    m_pAllocArray[1]= m_pAllocArray[2] = e1*e2;
    m_pAllocArray[3]=e2*e2;
}

template<class T> 
void TMatrix4<T>::setOuterProduct( TVector2<T>& v1, TVector2<T>& v2 )
{
    m_pAllocArray[0]=v1(0)*v2(0);
    m_pAllocArray[1]=v1(0)*v2(1);
    m_pAllocArray[2]=v1(1)*v2(0);
    m_pAllocArray[3]=v1(1)*v2(1);
}

template<class T> 
void TMatrix4<T>:: setAddOuterProduct( T* v )
{
    m_pAllocArray[0]+=v[0]*v[0];
    m_pAllocArray[1]+=v[0]*v[1];
    m_pAllocArray[2]+=v[0]*v[1];
    m_pAllocArray[3]+=v[1]*v[1];
}

template<class T> 
void TMatrix4<T>:: setAddOuterProduct( T e1, T e2 )
{
    m_pAllocArray[0]+=e1*e1;
    m_pAllocArray[1]+=e1*e2;
    m_pAllocArray[2]+=e1*e2;
    m_pAllocArray[3]+=e2*e2;
}


template<class T> 
void TMatrix4<T>:: setSubtractOuterProduct( T* v )
{
    m_pAllocArray[0]-=v[0]*v[0];
    m_pAllocArray[1]-=v[0]*v[1];
    m_pAllocArray[2]-=v[0]*v[1];
    m_pAllocArray[3]-=v[1]*v[1];
}

template<class T> 
void TMatrix4<T>:: setSubtractOuterProduct( T e1, T e2 )
{
    m_pAllocArray[0]-=e1*e1;
    m_pAllocArray[1]-=e1*e2;
    m_pAllocArray[2]-=e1*e2;
    m_pAllocArray[3]-=e2*e2;
}

template<class T> 
void TMatrix4<T>:: setSubtractOuterProduct( TVector2<T>& v )
{
    m_pAllocArray[0]-=v(0)*v(0);
    m_pAllocArray[1]-=v(0)*v(1);
    m_pAllocArray[2]-=v(0)*v(1);
    m_pAllocArray[3]-=v(1)*v(1);
}

template<class T>
void TMatrix4<T>:: setScaleEq( T val )
{
    for ( int i=0; i<4; i++ )
		m_pAllocArray[i]*=val;
}

template<class T>
void TMatrix4<T>:: setDivideEq( T val )
{
    for ( int i=0; i<4; i++ )
		m_pAllocArray[i]/=val;
}

template<class T> 
T TMatrix4<T>::getDet()
{
    return( m_pAllocArray[0]*m_pAllocArray[3]-m_pAllocArray[1]*m_pAllocArray[2] );
}

template<class T> 
int TMatrix4<T>::setInvert( TMatrix4<T> &m )
{
    TMatrix4 tmp( m.getData() );
    T det = m.getDet();
    if ( det == 0 )
		return 1;
    else {
		m_pAllocArray[0]=tmp(3)/det;
		m_pAllocArray[1]=-tmp(2)/det;
		m_pAllocArray[2]=-tmp(1)/det;
		m_pAllocArray[3]=tmp(0)/det;
		return 0;
    }
}

template<class T>
int TMatrix4<T>::setInvertEq()
{
    return setInvert( *this );
}

template<class T>
int
TMatrix4<T>::
operator==( const TMatrix4<T>& m ) 
{
    for ( register int i=0; i<4; i++ )
		if ( m_pAllocArray[i]!=m(i) ) return 0;
		return 1;
}



#endif /* _TMatrix4_H_ */



