/***************************************************************************
*
* Copyright 1997 by  the  Massachusetts Institute  of Technology.   All
* rights reserved.
*
* Developed  by the Perceptual Computing Section
* at the Media Laboratory, MIT, Cambridge, Massachusetts.
*
* Permission to use, copy,  or modify this software and  its documentation
* for  educational  and  research purposes only and without fee  is hereby
* granted, provided  that this copyright notice and the original authors's
* names appear  on all copies and supporting documentation.  If individual
* files are  separated from  this  distribution directory  structure, this
* copyright notice must be included.  For any other uses of this software,
* in original or  modified form, including but not limited to distribution
* in whole or in  part, specific  prior permission  must be  obtained from
* MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
*   basis  of  a commercial  software  or  hardware product  without first
* obtaining appropriate licenses  from MIT.  MIT. makes no representations
* about the suitability of this  software for any purpose.  It is provided
* "as is" without express or implied warranty.
*
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// Filename	: VDibBase.cpp
// Created	: Tuesday, June 3rd, 1997
// Description: Base class implementation. VDib and VDibSection will be derived from here.
// 
//  Vasanth Philomin 1997, vasi@cs.umd.edu
///////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "VDibBase.h"
#include <math.h>

int VDibBase::fCols[kBMMaxCols];
int VDibBase::fRows[kBMMaxRows];


CSize VDibBase::GetDimensions()
{
	return CSize(m_bmHeader.bmiHeader.biWidth, abs(m_bmHeader.bmiHeader.biHeight));
}


//
//    RotatedCopy
//

ErrorNumber VDibBase::RotatedCopy (VDibBase* dest, CRect* destRect,
								   ULONG cx, ULONG cy,
								   int width, int height, float angle)
{
	ErrorNumber     err       = kErrNoErr;
	CRect           rect      = (destRect != NULL) ?
		*destRect : dest->GetValidRect ();
	DWORD      *base;
	BYTE          *linePtr;
	DWORD       offset    = dest->LineOffset();
	
	// iWidth should be fSpec.lineOffset in PIXELS but as this function only
	// works with 4 byte pixels, iWidth = fSpec.width also works!!!!
	
	ULONG          iWidth    = Width();
	int          midh      = height >> 1;
	int          midw      = width >> 1;
	int        dx        = (int) (cos(angle)*10000.0 + 0.5);
	int        dy        = (int) (sin(angle)*10000.0 + 0.5);
	short int       quadrant;
	int            *cols      = fCols;
	int            *rows      = fRows;
	int             i, j, sx, sy, temp;
	
	// verify size of destination
	if ((rect != GetValidRect()) ||
        !InView (cx, cy, width, height, angle))
		// error on destination size
		return kErrWrongSize;
	
	// verify source and destination pixel encoding
	if ( (PixelSize() != 32) || ( dest->PixelSize() != 32 ) )
		err = kErrCantHandleEncoding;
	
	// compute the base of detination memory
	linePtr = (LPBYTE)(dest->GetBits ())  + rect.top * offset + 
		rect.left * 4;
	
	// Make the angle is positive
	if (angle < 0)
		angle += 2.0 * M_PI * (-((int) (angle/(2.0*M_PI))) + 1);
	
	// Figure out which quadrant it lies in
	quadrant = angle * (4.0 / M_PI);
	
	switch (quadrant & 0x7)
	{
    case 0:
		// Get row and column offsets for window
		bham_array_pos(rows, dx, dy, midw);
		bham_array_neg(cols, dx, dy, midh);
		
		// Turn the row offsets and related factors into memory addresses.
		for (i = 0; i < width; i++) rows[i] *= iWidth;
		sy = ((cy-midh) * iWidth);
		
		// Go through and put the data into the image buffer.
		temp = cx - midw;
		
		for (j = 0; j < height; j++, sy += iWidth)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sx = temp + cols[j]; i < width; i++, sx++)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy + rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 1:
		bham_array_pos(cols, dy, dx, midh);
		bham_array_neg(rows, dy, dx, midw);
		
		for (i = 0; i < height; i++) cols[i] *= iWidth;
		
		temp = (cy-midw) * iWidth;
		sx = cx + midh;
		
		for (j = 0; j < height; j++, sx -= 1)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sy = temp + cols[j]; i < width; i++, sy += iWidth)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy - rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 2:
		bham_array_neg(cols, dy, -dx, midh);
		bham_array_pos(rows, dy, -dx, midw);
		
		for (i = 0; i < height; i++) cols[i] *= iWidth;
		
		temp = (cy-midw) * iWidth;
		sx = cx + midh;
		
		for (j = 0; j < height; j++, sx -= 1)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sy = temp + cols[j]; i < width; i++, sy += iWidth)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy - rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 3:
		bham_array_neg(rows, -dx, dy, midw);
		bham_array_pos(cols, -dx, dy, midh);
		
		for (i = 0; i < width; i++) rows[i] *= iWidth;
		sy = (cy + midh)* iWidth;
		
		temp = cx+midw;
		
		for (j = 0; j < height; j++, sy -= iWidth)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sx = temp - cols[j]; i < width; i++, sx--)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy - rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 4:
		bham_array_neg(rows, -dx, -dy, midw);
		bham_array_pos(cols, -dx, -dy, midh);
		
		for (i = 0; i < width; i++) rows[i] *= iWidth;
		sy = (cy + midh)* iWidth;
		
		temp = cx + midw;
		
		for (j = 0; j < height; j++, sy -= iWidth)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sx = temp + cols[j]; i < width; i++, sx--)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy + rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 5:
		bham_array_neg(rows, -dy, -dx, midw);
		bham_array_pos(cols, -dy, -dx, midh);
		
		for (i = 0; i < height; i++) cols[i] *= iWidth;
		temp = (cy + midw) * iWidth;
		
		sx = cx - midh;
		
		for (j = 0; j < height; j++, sx += 1)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sy = temp - cols[j]; i < width; i++, sy -= iWidth)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy + rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 6:
		bham_array_neg(rows, -dy, dx, midw);
		bham_array_pos(cols, -dy, dx, midh);
		
		for (i = 0; i < height; i++) cols[i] *= iWidth;
		temp = (cy + midw) * iWidth;
		
		sx = cx - midh;
		
		for (j = 0; j < height; j++, sx += 1)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sy = temp + cols[j]; i < width; i++, sy -= iWidth)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy - rows[i]);
			
			linePtr += offset;
		}
		break;
		
    case 7:
		bham_array_neg(rows, dx, -dy, midw);
		bham_array_pos(cols, dx, -dy, midh);
		
		for (i = 0; i < width; i++) rows[i] *= iWidth;
		temp = cx - midw;
		
		sy = (cy - midh) * iWidth;
		
		for (j = 0; j < height; j++, sy += iWidth)
		{
			base = (LPDWORD)linePtr;
			
			for (i = 0, sx = temp + cols[j]; i < width; i++, sx++)
				*base++ = *(((LPDWORD)m_pBits) + sx + sy + rows[i]);
			
			linePtr += offset;
		}
		break;
		
    default:
		break;
  }
  
  return err;
}


////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//                         protected members definition                       //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

//
//    InView
//

inline
BOOL VDibBase::InView (int cx, int cy, int w, int h)
{
	w = (++w)>>1;
	h = (++h)>>1;
	
	return (BOOL)(((cx-w)>=0) && ((cx+w) <= Width())
		&& ((cy-h)>=0) && ((cy+h) <= abs(Height())));
}



//
//    InView
//

inline
BOOL	VDibBase::InView (int cx, int cy, int w, int h, float angle)
{
	double    c = fabs (cos(angle));
	double    s = fabs (sin(angle));
	// VERY IMPORTANT NOTE!!! This factor is the ratio of aspect between
	// the rotated pixel and the source image pixel
	double    f = 1/max( c, s );
	
	return InView (cx, cy, (int)ceil((c * w + s * h)*f), (int)ceil((c * h + s * w)*f));
}




//
//    bham_array_pos
//

inline
void VDibBase::bham_array_pos (register int *ys, short int dx, short int dy,
							   short int mid)
{
	short int             i, y = 0;
	register short int    err = (dy>>1) - (dx>>1);
	
	ys += mid;
	
	*ys = 0;
	for ( i = 1; i <= mid; i++)
	{
		err += dy;
		if (err > 0)
		{
			y++;
			err -= dx;
		}
		ys[-i] = - (ys[i] = y);
	}
}




//
//    bham_array_neg
//

inline
void VDibBase::bham_array_neg (register int *ys, short int dx, short int dy, 
							   short int mid)
{
	short int               i, y = 0;
	short int      err = (dy>>1) - (dx>>1);
	
	ys += mid;
	
	*ys = 0;
	for (i = 1; i <= mid; i++)
	{
		err += dy;
		if (err > 0)
		{
			y--;
			err -= dx;
		}
		ys[-i] = - (ys[i] = y);
	}
}

