#ifndef _ODXDEF_H_
#define _ODXDEF_H_

/**

  This file contains ODX Structure Definitions.
  
	History:
	
	  1.00 July 21st, 1994, ADR
	  
		1.01 May 23rd, 1995, ADR
        Added the definition of VKD (Video Keying Device) in the KEYING.*
        section.
		
		  1.02 May 29th, 1995, ADR
		  Added the ACQATTRIBUTE_RLEN bit.
		  
			1.03 Sept 20th, 1995, MM
			Added the defines for board IDs.
			
			  1.04 October 13th, 1995, BSD
			  added CYCLEDEF
			  
				1.05  October 13th, 1995, RH
				-  CYCLEDEF was added after the final "#endif"
				
				  1.06  October 18th, 1995
				  - added ACQATTRIBUTE_FBCYCLE 0x004
				  
					1.07  October 23rd, 1995, ADR
					- Added ALU_MIN_MINUS definition for fbalu/fbgrabalu.
					
					  1.08  October 25th, 1995, ADR
					  - Removed startindex from the CYCLEDEF Struct.
					  
						1.09  November 1st, 1995, ADR
						- Added the definitions: PACK_PAGE, Y_PAGE, U_PAGE, V_PAGE.
						
						  1.10  April 18th, 1996, ADR
						  - _VIDIN_FORMAT std field is now 5.
						  
							$Log: odxdef.h $
							Revision 1.13  1997/01/18 16:13:18  ADR
							- added new parameters to CAMDEF structure (Version 5).
							Revision 1.12  1996/12/20 17:03:54  ADR
							- Added definitions of Device ID for all of the ODxxx Devices.
							Revision 1.11  1996/09/25 17:29:20  ADR
							- Use CORSTD.H instead of STD.H
							Revision 1.10  1996/09/10 18:22:26  MM
							Added the EVENTMODE_START_OF_FIELD mode for host interrrupts.
							Renamed typedef EVENTFCT to PEVENTFCT to avoid confusion.
							Revision 1.9  1996/08/14 11:13:00  SINBAL
							removed variables reffering to the RTP module.  These 
							variable types are define in the header rtp.h. 
							Revision 1.8  1996/07/25 12:37:22  Balbir
							added function rtpgetfb() and rtpsetfct() to for the RTP.
							Revision 1.7  1996/07/19 16:28:42  ADR
							- Added the EVENTMODE definitions and EVENTFCT typedef.
							Revision 1.6  1996/04/18 12:35:46  ADR
							- _VIDIN_FORMAT std field is now 5.
							
**/
#include "corstd.h"
#include "odx.h"
#include "opr.h"

/****************************************************************
BOARD IDs
*****************************************************************/
#define OD300     0
#define OD20      1
#define ODVGA     2
#define OD500     3
#define OD500MS   4
#define OD300RLE  5
#define OD64      6
#define ODUCP     7
#define ODNULL    8
#define ODQUAD    9
#define ODTCX     10
#define ODF64     11
#define ODTCI     12
#define ODF64P    13


/****************************************************************
OPR RELATED DEFINITIONS
*****************************************************************/

/******************************
ACQATTRIBUTE
*******************************/
typedef union
{
	struct
	{
		UINT histo     :1;
		UINT rlen      :1;
		UINT fbcycle   :1; 
		UINT reserved  :14;
	} bit;
	ODXPARM value; 
} _ACQATTRIBUTE;

/* ACQATTRIBUTE Masks   */
#define ACQATTRIBUTE_HISTO 0x0001
#define ACQATTRIBUTE_RLEN  0x0002
#define ACQATTRIBUTE_FBCYCLE 0x004


/******************************
ACQMODE
*******************************/
typedef union
{
	struct
	{
		UINT sfld      :2;
		UINT ms        :1;
		UINT reserved  :13;
	} bit; 
	ODXPARM value; 
} _ACQMODE;

#define  ACQMODE_SFLD_NEXT 0
#define  ACQMODE_SFLD_ODD  1
#define  ACQMODE_SFLD_EVEN 2

#define  ACQMODE_MS_AUTOMATIC 0
#define  ACQMODE_MS_MANUAL    1


/******************************
ACQSTATUS
*******************************/
typedef union
{
	struct
	{
		UINT fga       :1;
		UINT fgp       :1;
		UINT blk       :1;
		UINT vfld      :2;
		UINT fsts      :2;
		UINT reserved  :9;
	} bit;
	ODXPARM value; 
} _ACQSTATUS;

#define ACQSTATUS_VFLD_NI  0     /* Non-Interlaced          */
#define ACQSTATUS_VFLD_IOF 1     /* Interlaced Odd Field    */
#define ACQSTATUS_VFLD_IEF 2     /* Interlaced Even Field   */
#define ACQSTATUS_VFLD_SL  3     /* Synchronization Lost    */

#define ACQSTATUS_FSTS_EMPTY        0
#define ACQSTATUS_FSTS_HALF_FULL    1
#define ACQSTATUS_FSTS_ALMOST_FULL  2
#define ACQSTATUS_FSTS_FULL         3


/******************************
AIC.*
*******************************/
typedef struct
{
	ODXPARM  red;
	ODXPARM  green;
	ODXPARM  blue; 
} AICPAGE;

typedef struct
{
	ODXPARM  saturation;
	ODXPARM  hue;
	AICPAGE  brightness;
	AICPAGE  contrast;
	AICPAGE  offset;
	AICPAGE  range; 
} AICDEF;

/******************************
BIN.OPM
(see opr.h)
*******************************/


/******************************
CGA.OPM
(see opr.h for ppop definitions)
*******************************/
typedef union
{
	struct
	{
		UINT ppop      :4;
		UINT t         :1;
		UINT aclip     :1;
		UINT reserved  :10;
	} bit;
	ODXPARM value; 
} _CGA_OPM;


/******************************
DSPMODE
*******************************/
typedef union
{
	struct
	{
		UINT sfld      :2;
		UINT ms        :1;
		UINT reserved  :13;
	} bit;
	ODXPARM value; 
} _DSPMODE;

#define  DSPMODE_SFLD_NEXT 0
#define  DSPMODE_SFLD_ODD  1
#define  DSPMODE_SFLD_EVEN 2

#define  DSPMODE_MS_AUTOMATIC 0
#define  DSPMODE_MS_MANUAL    1


/******************************
ERRNO
(see opr.h)
*******************************/

/******************************
EVENTMODE
*******************************/
#define EVENTMODE_END_OF_FRAME   0x0001
#define EVENTMODE_END_OF_GRAB    0x0002
#define EVENTMODE_END_OF_ODD     0x0004
#define EVENTMODE_END_OF_EVEN    0x0008
#define EVENTMODE_END_OF_FIELD   0x0010
#define EVENTMODE_START_OF_FIELD 0x0020

/******************************
FBTYPE
(see opr.h)
*******************************/


/******************************
FGON
(see opr.h)
*******************************/


/******************************
ICR
*******************************/
typedef union
{
	struct
	{
		UINT cams      :2;
		UINT master    :1;
		UINT colm      :1;
		UINT autosync  :1;
		UINT ssync     :1;
		UINT vss       :3;
		UINT yc        :1;
		UINT ct        :1;
		UINT rd        :1;
		UINT gd        :1;
		UINT bd        :1;
		UINT dc        :1; 
		UINT reserved  :1;
	} bit;
	ODXPARM value; 
} _ICR;

#define ICR_DEVICE_SLAVE   0
#define ICR_DEVICE_MASTER  1

#define ICR_COMPOSITE_SYNC 0
#define ICR_SEPARATE_SYNC  1

#define ICR_VSS_CVS  0     /* Composite Video Signal  */
#define ICR_VSS_RVS  1     /* Red Video Signal        */
#define ICR_VSS_GVS  2     /* Green Video Signal      */
#define ICR_VSS_BVS  3     /* Blue Video Signal       */
#define ICR_VSS_CS   4     /* Composite Sync          */


/******************************
ILUTFORM
*******************************/
typedef union
{
	struct
	{
		UINT no         :4;
		UINT nib        :4; 
		UINT type       :2; 
		UINT reserved   :6;
	} bit;
	ODXPARM value; 
} _ILUTFORM;

#define ILUTFORM_TYPE_MLUT    0  /* Monochrome Lut */
#define ILUTFORM_TYPE_CLUTNI  2  /* Color Lut Non-Interlaced   */
#define ILUTFORM_TYPE_CLUTI   3  /* Color Lut Interlaced       */

/******************************
KEYING.*
*******************************/
typedef union
{
	struct
	{
		UINT vks       :1;
		UINT vkm       :2;
		UINT vkd       :1;
		UINT reserved  :12;
	} bit;
	ODXPARM value; 
} _KEYING_MODE;

#define KM_VKS_OVERLAY  0
#define KM_VKS_VGA      1

#define KM_VKM_VKS_ONLY 0
#define KM_VKM_KEY      1
#define KM_VKM_FB_ONLY  2

#define KM_VKD    0x08


/******************************
OCR
*******************************/
typedef union
{
	struct
	{
		UINT dlut      :1;
		UINT dpm       :1;
		UINT dss       :2;
		UINT dcs       :2;
		UINT rse       :1;
		UINT gse       :1;
		UINT bse       :1;
		UINT opze      :1;
		UINT reserved  :6;
	} bit;
	ODXPARM value; 
} _OCR; 

#define OCR_DSS_SLAVE_ACQ  0
#define OCR_DSS_SLAVE_VGA  1
#define OCR_DSS_MASTER     2

#define OCR_DCS_ACQ        0
#define OCR_DCS_VGA        1
#define OCR_DCS_MASTER     2


/******************************
ODI
*******************************/
typedef union
{
	struct
	{
		UINT index     :2;
		UINT type      :3;
		UINT read      :1;
		UINT dsp       :1;
		UINT acq       :1;
		UINT ovl       :1;
		UINT reserved  :7;
	} bit;
	ODXPARM value; 
} _ODI;

#define ODI_TYPE_MONO         0
#define ODI_TYPE_RGB          0x0004
#define ODI_READ_WRITE        0x0000
#define ODI_READ_ONLY         0x0020
#define ODI_WITH_DISPLAY      0x0040
#define ODI_WITH_ACQUISITION  0x0080
#define ODI_INDEX_MASK        0x0003
#define ODI_TYPE_MASK         0x001c
#define ODI_READ_MASK         0x0020
#define ODI_DISPLAY_MASK      0x0040
#define ODI_ACQUISITION_MASK  0x0080

/******************************
OLUTFORM
*******************************/
typedef union
{
	struct
	{
		UINT no         :4;
		UINT nib        :4; 
		UINT type       :2; 
		UINT reserved   :6;
	} bit;
	ODXPARM value; 
} _OLUTFORM;

#define OLUTFORM_TYPE_MLUT    0  /* Monochrome Lut */
#define OLUTFORM_TYPE_CLUTNI  2  /* Color Lut Non-Interlaced   */
#define OLUTFORM_TYPE_CLUTI   3  /* Color Lut Interlaced       */


/******************************
PFBPAGE
*******************************/
#define RED_PAGE     0
#define GREEN_PAGE   1
#define BLUE_PAGE    2
#define PACK_PAGE    3
#define Y_PAGE       0
#define U_PAGE       1
#define V_PAGE       2

/******************************
VID
*******************************/
typedef union
{
	struct
	{
		UINT VersionID :12;
		UINT BetaID    :4;
	} bit;
	ODXPARM value; 
} _VID;


/******************************
VIDIN_FORMAT
*******************************/
typedef union
{
	struct
	{
		UINT vfld      :2;
		UINT std       :5;
		UINT fo        :1; 
		UINT reserved  :8;
	} bit;
	ODXPARM value; 
} _VIDIN_FORMAT;

#define VIDIN_VFLD_NI   0     /* Non-Interlaced          */
#define VIDIN_VFLD_OFI  1     /* Odd field, Interlaced   */
#define VIDIN_VFLD_EFI  2     /* Even field, Interlaced  */
#define VIDIN_VFLD_FFI  3     /* Full Frame, Interlaced  */


/****************************************************************
ODX FUNCTION RELATED DEFINITIONS
*****************************************************************/

/******************************
Relative Direction Symbols
(see opr.h)
*******************************/


/******************************
CNTRNEXT
*******************************/
typedef struct
{
	WORD flag;    
	WORD xmin;        
	WORD ymin;
	WORD xmax;
	WORD ymax;   
	WORD nxdir;       
	WORD nydir;               
	WORD nxydir;
	LONG area;        
	LONG xarea;       
	LONG yarea; 
} CNTRFEAT;

/* Contour Feature Flag Mask */
#define F_BOX        0x0001
#define F_PERIM      0x0002
#define F_AREA       0x0004
#define F_CENTROID   0x0008


/******************************
DEFCAMERA
*******************************/
/*
Version 1: 1st release
Version 2: Added the wDCRestoration Parameter.
Version 3: Added the Name Parameter.
Version 4: Added the FilterEnable
Version 5: Added: AcqAFilterFreq...ResetWidth               

*/
#define CAMDEF_VERSION  5
typedef struct
{
	WORD Version;                /* Version of this structure  */
	WORD CameraConnector;        
	WORD VidinFormat;
	WORD VidinHsfi;  
	WORD AcquisitionSyncSource;
	WORD CameraType;
	WORD NumberOfBits;
	WORD Range;
	WORD Offset;
	WORD AcquisitionType;
	WORD DCRestoration;
	char Name[32];
	WORD AAFilterEnable;
	DWORD AAFilterFreq;              
	WORD ConnectorType;
	WORD ChannelType;
	WORD FrameInteg;
	WORD FrameIntegPol;
	WORD FrameIntegMethod;
	WORD TimeInteg;
	WORD TimeIntegPol;
	WORD TimeIntegMethod;
	WORD Trigger;
	WORD TriggerPol;
	WORD TriggerMethod;
	DWORD TriggerWidth;
	WORD TriggerExpose;
	WORD Reset;
	WORD ResetPol;
	WORD ResetMethod;
	DWORD ResetWidth;
	WORD PClkMult;               // Pixel Clock Multiplier to feed a Camera
	WORD SyncSepType;
	WORD WriteEnable;
	WORD WriteEnablePol;
	WORD WriteEnableMethod; 
} CAMDEF;

/* Acquisition Type Definitions  */
#define CAMDEF_ACQUISITION_TYPE_OTHER        0
#define CAMDEF_ACQUISITION_TYPE_RS170_NTSC   1
#define CAMDEF_ACQUISITION_TYPE_CCIR_PAL     2

/* Connector Type Definitions */
#define CAMDEF_CONNECTOR_TYPE_GENERIC  0
#define CAMDEF_CONNECTOR_TYPE_SCSI_50  1
#define CAMDEF_CONNECTOR_TYPE_SCSI_68  2

/* Channel Type Definitions   */
#define CAMDEF_CHANNEL_TYPE_SINGLE     0
#define CAMDEF_CHANNEL_TYPE_DUAL       1 

/* Sync Separate Type Definitions   */
#define CAMDEF_SYNCSEP_TYPE_HV_SYNC    0     /* Hsync/Vsync */
#define CAMDEF_SYNCSEP_TYPE_LF_ENABLE  1     /* Line/Frame Enable */

/******************************
DEFVIDIN
*******************************/
typedef struct
{
	WORD  mode;
	WORD  std;
	WORD  hsfi;
	WORD  hsf;
	WORD  hsync;
	WORD  hblank;
	WORD  hpix;
	WORD  htotal;
	WORD  vsync;
	WORD  vblank;
	WORD  vpix;
	WORD  vtotal;
	char  name[32];
	WORD  hfp;
	WORD  vfp;
	DWORD pixclk; 
	WORD  hpolarity;
	WORD  vpolarity;
	WORD  dcrps;
	WORD  dcrpw;
} VIDEF;

typedef union
{
	struct
	{
		UINT ni        :1;
		UINT ai        :1;
		UINT clk       :1;
		UINT prpm      :1;
		UINT dif       :1; 
		UINT reserved  :11;
	} bit;
	WORD value; 
} _VIDEF_MODE;

/* Video Mode Masks  */
#define VIDEF_MODE_NI   0x0001   /* Non-Interlaced                */
#define VIDEF_MODE_AI   0x0002   /* Acquisition Interface         */
#define VIDEF_MODE_CLK  0x0004   /* Acquisition Clock             */
#define VIDEF_MODE_PRPM 0x0008   /* Pixel Rate Programming Mode   */
#define VIDEF_MODE_DIF  0x0010   /* Differential Input Camera     */


/******************************
DEFVIDOUT
*******************************/
/* Structure for 'defvidout' function */
typedef struct
{
	WORD  mode;
	WORD  std;
	WORD  hsfi;
	WORD  hsf;
	WORD  hsync;
	WORD  hblank;
	WORD  hpix;
	WORD  htotal;
	WORD  vsync;
	WORD  vblank;
	WORD  vpix;
	WORD  vtotal;
	char  name[32];
	WORD  hfp;
	WORD  vfp;
	DWORD pixclk;
	WORD  hpolarity;
	WORD  vpolarity;
	WORD  ire; 
} VODEF;

typedef union
{
	struct
	{
		UINT ni        :1;
		UINT df        :2;
		UINT prpm      :1;
		UINT reserved  :12;
	} bit;
	WORD value; 
} _VODEF_MODE;

/* Video Mode Masks  */
#define VODEF_MODE_NI   0x0001    /* Non-Interlaced   */
#define VODEF_MODE_DF   0x0006    /* Display Format   */
#define VODEF_MODE_PRPM 0x0008    /* Pixel Rate Programming Mode  */


/******************************
DRIVERID
*******************************/
typedef struct
{
	WORD DriverID;
	char Name[64]; 
} DRIVERID;


/******************************
FBALU/FBGRABALU
*******************************/
#define ALU_ADD         0
#define ALU_MINUS       1  
#define ALU_ABS_MINUS   2
#define ALU_AND         3
#define ALU_OR          4
#define ALU_XOR         5
#define ALU_MIN         6
#define ALU_MAX         7
#define ALU_ADD_DIV2    8
#define ALU_MIN_MINUS   9  

/******************************
FORMAT
*******************************/
typedef union
{
	struct
	{
		UINT mwi       :3;
		UINT mhi       :3;
		UINT reserved  :10;
	} bit;
	ODXPARM value; 
} _FORMAT_MODE;

#define FORMAT_256   0
#define FORMAT_512   1
#define FORMAT_768   2
#define FORMAT_1024  3
#define FORMAT_1280  4
#define FORMAT_1536  5
#define FORMAT_2048  6
#define FORMAT_4096  7


/******************************
MATHOP: op
(see opr.h)
*******************************/


/******************************
MORPHO: mode
(see opr.h)
*******************************/


/******************************
PIXPTN
*******************************/
typedef union
{
	struct
	{
		UINT xlen    :4;
		UINT ylen    :4;
		UINT xzf     :4;
		UINT yzf     :4;
	} bit;
	ODXPARM value;
} PIXPTN;


/******************************
RLEN
*******************************/
typedef struct
{
	WORD pos;          /* position of run */
	WORD cnt;          /* number of pixels in run */
} GRP;


/******************************
SETEVENTFCT
*******************************/
typedef void (CALLBACK *PEVENTFCT)(ODXPARM);

/******************************
VENDORID
*******************************/
typedef struct
{
	WORD VendorID;
	DWORD SerialID;
	char Name[64]; 
} VENDORID;

/******************************
DEFFBCYCLE
*******************************/
#define   CYCLEDEF_VERSION 1 
typedef struct
{
	WORD version;              /*  Version of this structure  */
	WORD  fbstart;
	WORD  fbend;
	WORD  dx; 
	WORD  dy;
	WORD  flag; 
} CYCLEDEF;

#endif  /* _ODXDEF_H_ */
